import React, { useMemo } from 'react';
import DomPurify from 'dompurify';

const defaultTags = [
  'b',
  'strong',
  'sub',
  'sup',
  'br',
  'a',
  'p',
  'ul',
  'li',
  'ol',
];

const elementClassNameMapping = {
  a: 'ds-link',
  p: 'ds-body',
  b: 'ds-bold',
  strong: 'ds-bold',
  sub: 'ds-sub',
  ul: 'ds-list ds-list--bullet',
  ol: 'ds-list ds-list--number',
  li: 'ds-list__item',
};

if (typeof window !== 'undefined') {
  DomPurify.addHook('afterSanitizeAttributes', function (node) {
    if (node.tagName.toLowerCase() in elementClassNameMapping) {
      node.setAttribute(
        'class',
        elementClassNameMapping[node.tagName.toLowerCase()]
      );
    }
    // set all elements owning target to target=_blank
    if ('target' in node) {
      node.setAttribute('target', '_blank');
      node.setAttribute('rel', 'noopener');
    }
  });
}

export const getClosingTags = (htmlString) => {
  const tags = htmlString.match(/<[^>]+\/?>/g);
  const stack: string[] = [];
  if (!tags) {
    return stack;
  }
  for (const tag of tags) {
    if (tag.startsWith('</') && tag.endsWith('>')) {
      const tagName = tag.replace('</', '<');
      // @ts-ignore
      const openingTag = stack.findLastIndex((element) => element === tagName);
      if (openingTag !== -1) {
        stack.splice(openingTag, 1);
      }
    } else {
      stack.push(tag.split(' ').length > 1 ? `${tag.split(' ')[0]}>` : tag);
    }
  }
  return stack;
};

export interface SafeHTMLProps {
  content: string;
  tags?: string[];
  Tag?: string;
  style?: React.CSSProperties;
}

export const SafeHTML = ({
  content,
  tags,
  Tag = 'span',
  ...props
}: SafeHTMLProps) => {
  if (tags) {
    tags.push(...defaultTags);
  } else {
    tags = defaultTags;
  }
  const parsedHTML = useMemo(() => {
    return DomPurify.sanitize(content, {
      ALLOWED_TAGS: tags,
      ADD_ATTR: ['target'],
    });
  }, [content]);
  return (
    <Tag
      // @ts-ignore
      dangerouslySetInnerHTML={{
        __html: parsedHTML,
      }}
      {...props}
    />
  );
};

export default SafeHTML;
